/*

USC-Controller, 8-Bit out
        _ _
 RST --| V |-- VCC
   D --|   |-- USB+
 CLK --|   |--
 GND --|___|-- USB-

Reiny

*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include <util/delay.h>

#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>

#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"

volatile uint8_t *const Register = &PORTB;
volatile uint8_t *const Direction = &DDRB;
const uint8_t DataBit = 1 << 3;
const uint8_t ClkBit = 1 << 4;

uint8_t var = 1;
uint8_t dir = 0;

void SetShiftRegister(uint8_t val)
{
	*Direction |= DataBit | ClkBit;

	for (uint8_t i = 0; i < 8; ++i)
	{
		if (val & 1)
			*Register |= DataBit;
		else
			*Register &= ~DataBit;

		val >>= 1;
		*Register |= ClkBit;
		if (i != 7)
			*Register &= ~ClkBit;
	}
	_delay_us(20);
	*Register &= ~ClkBit;
}

USB_PUBLIC uchar usbFunctionSetup(uchar data[8])
{
	usbRequest_t    *rq = (void *)data;
	static uchar    replyBuf[2];

    usbMsgPtr = replyBuf;
    if(rq->bRequest == 0)
	{  /* ECHO */
        replyBuf[0] = rq->wValue.bytes[0];
        replyBuf[1] = rq->wValue.bytes[1];
        return 2;
    }
    if(rq->bRequest == 1)
	{  /* swap bytes */
        replyBuf[0] = rq->wValue.bytes[1];
        replyBuf[1] = rq->wValue.bytes[0];
        return 2;
    }
    if(rq->bRequest == 2)
	{
		SetShiftRegister(rq->wValue.bytes[0]);
    }
    return 0;
}

static void calibrateOscillator(void)
{
	uchar       step = 128;
	uchar       trialValue = 0, optimumValue;
	int         x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

    /* do a binary search: */
    do
	{
        OSCCAL = trialValue + step;
        x = usbMeasureFrameLength();    /* proportional to current real frequency */
        if(x < targetValue)             /* frequency still too low */
            trialValue += step;
        step >>= 1;
    }
	while(step > 0);
    /* We have a precision of +/- 1 for optimum OSCCAL here */
    /* now do a neighborhood search for optimum value */
    optimumValue = trialValue;
    optimumDev = x; /* this is certainly far away from optimum */
    for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++)
	{
        x = usbMeasureFrameLength() - targetValue;
        if(x < 0)
            x = -x;
        if(x < optimumDev)
		{
            optimumDev = x;
            optimumValue = OSCCAL;
        }
    }
    OSCCAL = optimumValue;
}
void usbEventResetReady(void)
{
	uint8_t oldsreg = SREG;
	cli();
    calibrateOscillator();
	SREG = oldsreg;
}
int main(void)
{
    wdt_enable(WDTO_1S);
    odDebugInit();
    PORTB = 0;          /* no pullups on USB pins */
/* We fake an USB disconnect by pulling D+ and D- to 0 during reset. This is
 * necessary if we had a watchdog reset or brownout reset to notify the host
 * that it should re-enumerate the device. Otherwise the host's and device's
 * concept of the device-ID would be out of sync.
 */
    DDRB = ~USBMASK;    /* set all pins as outputs except USB */
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    uint8_t i = 0;
    while(--i)
	{         /* fake USB disconnect for > 500 ms */
        wdt_reset();
        _delay_ms(2);
    }
    sei();
    usbDeviceConnect();
    usbInit();
    for(;;)
	{    /* main event loop */
        wdt_reset();
        usbPoll();
    }
    return 0;
}

